#!/usr/bin/perl
###############################################################################
#    Webmin Sysstats - mailq-lib.pl
#
#    Webmin Sysstats Module
#    Copyright (C) 2001 by Eric Gerbier
#    Bug reports to: dbouius@users.sourceforge.net or gerbier@users.sourceforge.net
#    $Id: mailq-lib.pl,v 1.5 2001/12/07 10:43:44 gerbier Exp $
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################

sub install_module {
	local ( $minfo, $defaults ) = ( $_[0], $_[1] );

	local (%my_config);

	if ( !-r $minfo->{'dbtables'} . "/mailq.rrd" ) {
		local ($cmd);

		$cmd = $config{'rrdtool'};
		$cmd .= " create " . $minfo->{'dbtables'} . "mailq.rrd";
		$cmd .= " --start 0 --step 60";
		$cmd .= " DS:mailq:GAUGE:100:0:U";
		$cmd .= " DS:sendmail:GAUGE:100:0:U";
		$cmd .= " DS:pop:GAUGE:100:0:U";
		$cmd .= " DS:imap:GAUGE:100:0:U";
		$cmd .= " RRA:AVERAGE:0.5:1:70";
		$cmd .= " RRA:AVERAGE:0.5:2:750";
		$cmd .= " RRA:AVERAGE:0.5:15:650";
		$cmd .= " RRA:AVERAGE:0.5:50:650";

		`$cmd`;
	}

	$my_config{'sample_rate'} = '1';
	$my_config{'mailq'} = join (
		',',                            $defaults->{'mailq_colour'},
		$defaults->{'sendmail_colour'}, $defaults->{'pop_colour'},
		$defaults->{'imap_colour'}
	);

	&write_config( $minfo, \%my_config, $defaults );

	&install_script( $minfo, 'mailq.sh' );

	return 1;
}

sub remove_module {
}

sub mod_check_runstop {
	local ( $minfo, $my_config, $defaults, $opt ) = ( $_[0], $_[1], $_[2] );

	return $my_config->{'runstop'};
}

sub mod_runstop {
	local ( $minfo, $my_config, $defaults, $opt ) = ( $_[0], $_[1], $_[2] );

	$my_config->{'runstop'} = $my_config->{'runstop'} ? '1' : '0';
	&write_config( $minfo, $my_config, $defaults );
}

sub mod_check_upgrade {
	local ( $minfo, $my_config, $defaults ) = ( $_[0], $_[1], $_[2] );
	return 0;
}

sub mod_upgrade {
	local ( $minfo, $my_config, $defaults ) = ( $_[0], $_[1], $_[2] );
}

sub graph_data {
	local ( $minfo, $my_config, $defaults, $opt ) = ( $_[0], $_[1], $_[2] );

	$rrdtool = $config{'rrdtool'};
	$sizex   = $config{'graph_sizex'};
	$sizey   = $config{'graph_sizey'};

	( $mailq_colour, $sendmail_colour, $pop_colour, $imap_colour ) =
	  split ( ',', $my_config{'mailq'} );

	$rrd_detals = " -l 0 -v 'mail'";
	$rrd_detals .=
	  " DEF:mailq=" . $minfo->{'dbtables'} . "mailq.rrd:mailq:AVERAGE";
	$rrd_detals .=
	  " DEF:sendmail=" . $minfo->{'dbtables'} . "mailq.rrd:sendmail:AVERAGE";
	$rrd_detals .= " DEF:pop=" . $minfo->{'dbtables'} . "mailq.rrd:pop:AVERAGE";
	$rrd_detals .= " DEF:imap=" . $minfo->{'dbtables'} . "mailq.rrd:imap:AVERAGE";
	$rrd_detals .= " COMMENT:'Type         Max     Min Average\\n'";

	$rrd_detals .= " LINE1:mailq#" . $mailq_colour . ":'mailqueue  '";
	$rrd_detals .= " GPRINT:mailq:MAX:'%6.2lf'";
	$rrd_detals .= " GPRINT:mailq:MIN:'%6.2lf'";
	$rrd_detals .= " GPRINT:mailq:AVERAGE:'%6.2lf\\n'";

	$rrd_detals .= " LINE1:sendmail#" . $sendmail_colour . ":'sendmail  '";
	$rrd_detals .= " GPRINT:sendmail:MAX:'%6.2lf'";
	$rrd_detals .= " GPRINT:sendmail:MIN:'%6.2lf'";
	$rrd_detals .= " GPRINT:sendmail:AVERAGE:'%6.2lf\\n'";

	$rrd_detals .= " LINE1:pop#" . $pop_colour . ":'pop  '";
	$rrd_detals .= " GPRINT:pop:MAX:'%6.2lf'";
	$rrd_detals .= " GPRINT:pop:MIN:'%6.2lf'";
	$rrd_detals .= " GPRINT:pop:AVERAGE:'%6.2lf\\n'";

	$rrd_detals .= " LINE1:imap#" . $imap_colour . ":'imap  '";
	$rrd_detals .= " GPRINT:imap:MAX:'%6.2lf'";
	$rrd_detals .= " GPRINT:imap:MIN:'%6.2lf'";
	$rrd_detals .= " GPRINT:imap:AVERAGE:'%6.2lf\\n'";

	foreach $graph ( 'hour', 'day', 'week', 'month' ) {
		if ( $graph eq 'hour' ) {
			$time  = "-3600";
			$title = "'By the hour'";
		}
		elsif ( $graph eq 'day' ) {
			$time  = "-86400";
			$title = "'Daily'";
		}
		elsif ( $graph eq 'week' ) {
			$time  = "-604800";
			$title = "'Weekly'";
		}
		elsif ( $graph eq 'month' ) {
			$time  = "-18748800";
			$title = "'Monthly'";
		}

		$common =
		  $rrdtool . " graph " . $minfo->{'tmpgname'} . "_" . $graph . ".png";
		$common .= " --start " . $time . " --end -60";
		$common .= " -h " . $sizey . " -w " . $sizex;
		$common .= " -a PNG --title=" . $title;

		$sig = " COMMENT:'\\n' COMMENT:\"Created: `date`\"";

		system( $common . $rrd_detals . $sig );
	}
}

sub mod_config {
	local ( $minfo, $my_config, $defaults ) = ( $_[0], $_[1], $_[2] );

	print "<table width=100%><tr><td><b>";
	print $text{'module_subtitle'};
	print "</b></td></tr></table>\n";

	print "<form action=save_config.cgi>\n";
	print "<input type=hidden name=idx value='$in{'idx'}'>\n";

	print "<table width='100%'>\n";
	print "<tr>";
	print "<td width='15%'>";
	print "<b>" . $text{'word_mailq'} . "</b>";
	print "</td>";

	print "<td width='35%'>";
	&option_colourselect( 'mailq', $mailq_colour, $defaults->{'mailq_colour'} );
	print "</td>";
	print "</tr>";

	print "<tr>";
	print "<td width='15%'>";
	print "<b>" . $text{'word_sendmail'} . "</b>";
	print "</td>";

	print "<td width='35%'>";
	&option_colourselect( 'sendmail', $sendmail_colour,
		$defaults->{'sendmail_colour'} );
	print "</td>";
	print "</tr>";

	print "<tr>";
	print "<td width='15%'>";
	print "<b>" . $text{'word_pop'} . "</b>";
	print "</td>";

	print "<td width='35%'>";
	&option_colourselect( 'pop', $pop_colour, $defaults->{'pop_colour'} );
	print "</td>";
	print "</tr>";

	print "<tr>";
	print "<td width='15%'>";
	print "<b>" . $text{'word_imap'} . "</b>";
	print "</td>";

	print "<td width='35%'>";
	&option_colourselect( 'imap', $imap_colour, $defaults->{'imap_colour'} );
	print "</td>";
	print "</tr>";
	print "</table>";
	print "<input type='submit' value='" . $text{'word_save'} . "'>";
	print "</form>\n";
}

sub mod_config_save {
	local ( $minfo, $my_config, $defaults ) = ( $_[0], $_[1], $_[2] );

	$my_config->{'mailq'} =
	  join ( ',',
		( $in{'mailq1_def'} eq 'default' ) ? $defaults->{'mailq_colour'}
		: $in{'mailq2_def'} ? $in{'mailq2_def'}
		: $in{'mailq1_def'},
		( $in{'sendmail1_def'} eq 'default' ) ? $defaults->{'sendmail_colour'}
		: $in{'sendmail2_def'} ? $in{'sendmail2_def'}
		: $in{'sendmail1_def'},
		( $in{'pop1_def'} eq 'default' ) ? $defaults->{'pop_colour'}
		: $in{'pop2_def'} ? $in{'pop2_def'}
		: $in{'pop1_def'},
		( $in{'imap1_def'} eq 'default' ) ? $defaults->{'imap_colour'}
		: $in{'imap2_def'} ? $in{'imap2_def'}
		: $in{'imap1_def'} );

	&write_config( $minfo, $my_config, $defaults );
}

sub write_config {
	local ( $minfo, $my_config, $defaults ) = ( $_[0], $_[1], $_[2] );

	local ( %new_config, %sh_config );

	$new_config{'sample_rate'} = $my_config->{'sample_rate'};
	$sh_config{'sample_rate'}  = $my_config->{'sample_rate'};
	$new_config{'runstop'}     = $my_config->{'runstop'};
	$new_config{'mailq'}       = $my_config->{'mailq'};

	if ( !$new_config{'runstop'} ) {
		$sh_config{'mailq'} = 'mailq';
	}

	unlink( $minfo->{'config_file'} );
	unlink( $minfo->{'config_dir'} . 'mailq.cfg' );

	&write_file( $minfo->{'config_file'}, \%new_config );
	&write_file( $minfo->{'config_dir'} . 'mailq.cfg', \%sh_config );
}

return 1;
